@@ -26,6 +26,9 @@ else:
         # An explicit test db name was given, is that as the base name
         TEST_DB_NAME = "{}_inner".format(TEST_DB_NAME)
 
+SECOND_DB_NAME = DB_NAME + '_second' if DB_NAME is not None else None
+SECOND_TEST_DB_NAME = TEST_DB_NAME + '_second' if DB_NAME is not None else None
+
 
 def get_db_engine():
     return _settings["ENGINE"].split(".")[-1]
